/* A safe weak reference to a datum that exists.
	It's safe because it never refers to any datum but the original.
	It's weak because it doesn't prevent garbage collection of the referenced datum.
	It references the datum through Get().

	Usage:

		Make a new safe weak reference to a datum:

			var safe_weak_reference/r = new(datum)

		Get the strong reference:

			r.Get()

		Check whether two safe weak references are pointing to the same datum:

			r1 ~= r2 // same
			r1 ~! r2 // different

		Check whether a safe weak reference points to some strong reference:

			r ~= datum

		Note: datum ~= r is not the same unless you add it to your own /datum/proc/operator~=() override.
*/

safe_weak_reference

	New(datum/datum)
		..()
		if(datum)
			_unsafe_weak_reference = "\ref[datum]"
			_discriminator = (datum._weak_reference_discriminator || (datum._weak_reference_discriminator = new))
			_exists = TRUE
		else
			_exists = FALSE

	proc
		Get()
			/* Get the datum that is being referenced, or null if it no longer exists.
			*/
			if(_exists)
				var datum/found = locate(_unsafe_weak_reference)
				if(found?._weak_reference_discriminator == _discriminator)
					return found
				else
					_exists = FALSE
			return null

		operator~=(value)
			/* Weak references are equivalent if they refer to the same datum.
				This can also be used to check equality of the referenced object.
			*/
			if(src == value)
				return TRUE
			if(istype(value, /safe_weak_reference))
				var safe_weak_reference/r = value
				value = r.Get()
			return Get() == value

	var
		tmp
			_unsafe_weak_reference
			// Built-in unsafe weak reference from the ref text macro.

			datum/_discriminator
			// Matches the referenced datum's weak reference discriminator.

			_exists
			// True until the datum is found to no longer exist.

datum
	var
		tmp
			datum/_weak_reference_discriminator
			// Matches the discriminator of any safe weak references referring to this.
